/**
 * DevExtreme (cjs/__internal/grids/grid_core/sticky_columns/m_sticky_columns.js)
 * Version: 24.2.1-alpha-24256-1935
 * Build date: Thu Sep 12 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.stickyColumnsModule = void 0;
var _const = require("./const");
var _dom = require("./dom");
var _utils = require("./utils");
const baseStickyColumns = Base => class extends Base {
    _addStickyColumnBorderLeftClass($cell, column, rowIndex) {
        let onlyWithinBandColumn = arguments.length > 3 && void 0 !== arguments[3] ? arguments[3] : false;
        let fixedPosition = arguments.length > 4 ? arguments[4] : void 0;
        const isFirstFixedCell = (0, _utils.isFirstFixedColumn)(this._columnsController, column, rowIndex, onlyWithinBandColumn, fixedPosition);
        if (isFirstFixedCell) {
            _dom.GridCoreStickyColumnsDom.addStickyColumnBorderLeftClass($cell, this.addWidgetPrefix.bind(this))
        }
    }
    _addStickyColumnBorderRightClass($cell, column, rowIndex) {
        let onlyWithinBandColumn = arguments.length > 3 && void 0 !== arguments[3] ? arguments[3] : false;
        let fixedPosition = arguments.length > 4 ? arguments[4] : void 0;
        const isLastFixedCell = (0, _utils.isLastFixedColumn)(this._columnsController, column, rowIndex, onlyWithinBandColumn, fixedPosition);
        if (isLastFixedCell) {
            _dom.GridCoreStickyColumnsDom.addStickyColumnBorderRightClass($cell, this.addWidgetPrefix.bind(this))
        }
    }
    _isStickyColumns() {
        var _this$_columnsControl;
        const stickyColumns = null === (_this$_columnsControl = this._columnsController) || void 0 === _this$_columnsControl ? void 0 : _this$_columnsControl.getStickyColumns();
        return true !== this.option("columnFixing.legacyMode") && !!stickyColumns.length
    }
    _renderCore(options) {
        super._renderCore(options);
        const $element = this.element();
        const isStickyColumns = this._isStickyColumns();
        _dom.GridCoreStickyColumnsDom.toggleStickyColumnsClass($element, isStickyColumns, this.addWidgetPrefix.bind(this))
    }
    _createCell(options) {
        const {
            column: column
        } = options;
        const {
            rowType: rowType
        } = options;
        const $cell = super._createCell(options);
        const isStickyColumns = this._isStickyColumns();
        if (isStickyColumns && column.fixed) {
            const rowIndex = "header" === rowType ? options.rowIndex : null;
            const fixedPosition = (0, _utils.getColumnFixedPosition)(column);
            _dom.GridCoreStickyColumnsDom.addStickyColumnClass($cell, column, this.addWidgetPrefix.bind(this));
            switch (fixedPosition) {
                case _const.StickyPosition.Right:
                    this._addStickyColumnBorderLeftClass($cell, column, rowIndex, false, _const.StickyPosition.Right);
                    break;
                case _const.StickyPosition.Sticky:
                    this._addStickyColumnBorderLeftClass($cell, column, rowIndex, true);
                    this._addStickyColumnBorderRightClass($cell, column, rowIndex, true);
                    break;
                default:
                    this._addStickyColumnBorderRightClass($cell, column, rowIndex, false, _const.StickyPosition.Left)
            }
        }
        return $cell
    }
    setStickyOffsets(rowIndex, offsets) {
        let columns = this.getColumns(rowIndex);
        let widths = this.getColumnWidths(void 0, rowIndex);
        const rtlEnabled = this.option("rtlEnabled");
        if (rtlEnabled) {
            columns = rtlEnabled ? [...columns].reverse() : columns;
            widths = rtlEnabled ? [...widths].reverse() : widths
        }
        columns.forEach(((column, columnIndex) => {
            if (column.fixed) {
                const visibleColumnIndex = rtlEnabled ? columns.length - columnIndex - 1 : columnIndex;
                const offset = (0, _utils.getStickyOffset)(columns, widths, columnIndex, offsets);
                if (offsets) {
                    offsets[column.index] = offset
                }
                const styleProps = (0, _utils.normalizeOffset)(offset);
                this.setCellProperties(styleProps, visibleColumnIndex, rowIndex)
            }
        }))
    }
    _resizeCore() {
        const isStickyColumns = this._isStickyColumns();
        super._resizeCore.apply(this, arguments);
        if (isStickyColumns) {
            this.setStickyOffsets()
        }
    }
};
const columnHeadersView = Base => class extends(baseStickyColumns(Base)) {
    setStickyOffsets() {
        const offsets = {};
        const rowCount = this.getRowCount();
        for (let rowIndex = 0; rowIndex < rowCount; rowIndex++) {
            super.setStickyOffsets(rowIndex, offsets)
        }
    }
    _createCell(options) {
        const $cell = super._createCell(options);
        const rowCount = this.getRowCount();
        const {
            column: column,
            rowIndex: rowIndex
        } = options;
        const isStickyColumns = this._isStickyColumns();
        const columnsController = this._columnsController;
        if (isStickyColumns && rowCount > 1) {
            const prevCellIsFixed = (0, _utils.prevColumnIsFixed)(columnsController, column, rowIndex);
            if (prevCellIsFixed) {
                _dom.GridCoreStickyColumnsDom.addColumnNoBorderClass($cell, this.addWidgetPrefix.bind(this))
            }
            if (null !== columnsController && void 0 !== columnsController && columnsController.isFirstColumn(column, rowIndex)) {
                _dom.GridCoreStickyColumnsDom.addFirstHeaderClass($cell, this.addWidgetPrefix.bind(this))
            }
        }
        return $cell
    }
    getContextMenuItems(options) {
        const {
            column: column
        } = options;
        const columnFixingOptions = this.option("columnFixing");
        let items = super.getContextMenuItems(options);
        if (options.row && "header" === options.row.rowType) {
            if (true === columnFixingOptions.enabled && column && column.allowFixing) {
                const onItemClick = params => {
                    switch (params.itemData.value) {
                        case "none":
                            this._columnsController.columnOption(column.index, "fixed", false);
                            break;
                        case "left":
                            this._columnsController.columnOption(column.index, {
                                fixed: true,
                                fixedPosition: "left"
                            });
                            break;
                        case "right":
                            this._columnsController.columnOption(column.index, {
                                fixed: true,
                                fixedPosition: "right"
                            });
                            break;
                        case "sticky":
                            this._columnsController.columnOption(column.index, {
                                fixed: true,
                                fixedPosition: "sticky"
                            })
                    }
                };
                const fixedPositionItems = [{
                    text: columnFixingOptions.texts.leftPosition,
                    value: "left",
                    disabled: column.fixed && (!column.fixedPosition || "left" === column.fixedPosition),
                    onItemClick: onItemClick
                }, {
                    text: columnFixingOptions.texts.rightPosition,
                    value: "right",
                    disabled: column.fixed && "right" === column.fixedPosition,
                    onItemClick: onItemClick
                }];
                if (true !== this.option("columnFixing.legacyMode")) {
                    fixedPositionItems.push({
                        text: columnFixingOptions.texts.stickyPosition,
                        value: "sticky",
                        disabled: column.fixed && (0, _utils.getColumnFixedPosition)(column) === _const.StickyPosition.Sticky,
                        onItemClick: onItemClick
                    })
                }
                items = items || [];
                items.push({
                    text: columnFixingOptions.texts.fix,
                    beginGroup: true,
                    items: fixedPositionItems
                }, {
                    text: columnFixingOptions.texts.unfix,
                    value: "none",
                    disabled: !column.fixed,
                    onItemClick: onItemClick
                })
            }
        }
        return items
    }
};
const rowsView = Base => class extends(baseStickyColumns(Base)) {};
const footerView = Base => class extends(baseStickyColumns(Base)) {};
const stickyColumnsModule = exports.stickyColumnsModule = {
    extenders: {
        views: {
            columnHeadersView: columnHeadersView,
            rowsView: rowsView,
            footerView: footerView
        }
    }
};
