/**
 * DevExtreme (cjs/__internal/ui/chat/chat.js)
 * Version: 24.2.1-alpha-24256-1935
 * Build date: Thu Sep 12 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = void 0;
var _component_registrator = _interopRequireDefault(require("../../../core/component_registrator"));
var _guid = _interopRequireDefault(require("../../../core/guid"));
var _renderer = _interopRequireDefault(require("../../../core/renderer"));
var _widget = _interopRequireDefault(require("../../core/widget/widget"));
var _chat_header = _interopRequireDefault(require("./chat_header"));
var _chat_message_box = _interopRequireDefault(require("./chat_message_box"));
var _chat_message_list = _interopRequireDefault(require("./chat_message_list"));

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _extends() {
    _extends = Object.assign ? Object.assign.bind() : function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}
const CHAT_CLASS = "dx-chat";
const TEXTEDITOR_INPUT_CLASS = "dx-texteditor-input";
class Chat extends _widget.default {
    _getDefaultOptions() {
        return _extends({}, super._getDefaultOptions(), {
            activeStateEnabled: true,
            focusStateEnabled: true,
            hoverStateEnabled: true,
            title: "",
            items: [],
            dataSource: null,
            user: {
                id: (new _guid.default).toString()
            },
            onMessageSend: void 0
        })
    }
    _init() {
        super._init();
        this._createMessageSendAction()
    }
    _initMarkup() {
        (0, _renderer.default)(this.element()).addClass("dx-chat");
        super._initMarkup();
        this._renderMessageBox();
        this._renderMessageList();
        const {
            title: title
        } = this.option();
        if (title) {
            this._renderHeader(title)
        }
    }
    _renderHeader(title) {
        const $header = (0, _renderer.default)("<div>");
        this.element().prepend($header.get(0));
        this._chatHeader = this._createComponent($header, _chat_header.default, {
            title: title
        })
    }
    _renderMessageList() {
        const {
            items: items = [],
            user: user
        } = this.option();
        const currentUserId = null === user || void 0 === user ? void 0 : user.id;
        const $messageList = (0, _renderer.default)("<div>");
        this.$element().prepend($messageList);
        this._messageList = this._createComponent($messageList, _chat_message_list.default, {
            items: items,
            currentUserId: currentUserId
        })
    }
    _renderMessageBox() {
        const {
            activeStateEnabled: activeStateEnabled,
            focusStateEnabled: focusStateEnabled,
            hoverStateEnabled: hoverStateEnabled
        } = this.option();
        const $messageBox = (0, _renderer.default)("<div>").appendTo(this.element());
        const configuration = {
            activeStateEnabled: activeStateEnabled,
            focusStateEnabled: focusStateEnabled,
            hoverStateEnabled: hoverStateEnabled,
            onMessageSend: e => {
                this._messageSendHandler(e)
            }
        };
        this._messageBox = this._createComponent($messageBox, _chat_message_box.default, configuration)
    }
    _createMessageSendAction() {
        this._messageSendAction = this._createActionByOption("onMessageSend", {
            excludeValidators: ["disabled", "readOnly"]
        })
    }
    _messageSendHandler(e) {
        var _this$_messageSendAct;
        const {
            text: text,
            event: event
        } = e;
        const {
            user: user
        } = this.option();
        const message = {
            timestamp: new Date,
            author: user,
            text: text
        };
        this.renderMessage(message);
        null === (_this$_messageSendAct = this._messageSendAction) || void 0 === _this$_messageSendAct || _this$_messageSendAct.call(this, {
            message: message,
            event: event
        })
    }
    _focusTarget() {
        const $input = (0, _renderer.default)(this.element()).find(".dx-texteditor-input");
        return $input
    }
    _optionChanged(args) {
        const {
            name: name,
            value: value
        } = args;
        switch (name) {
            case "activeStateEnabled":
            case "focusStateEnabled":
            case "hoverStateEnabled":
                this._messageBox.option(name, value);
                break;
            case "title":
                if (value) {
                    if (this._chatHeader) {
                        this._chatHeader.option("title", value)
                    } else {
                        this._renderHeader(value)
                    }
                } else if (this._chatHeader) {
                    this._chatHeader.dispose();
                    this._chatHeader.$element().remove()
                }
                break;
            case "user": {
                const author = value;
                this._messageList.option("currentUserId", null === author || void 0 === author ? void 0 : author.id);
                break
            }
            case "items":
            case "dataSource":
                this._messageList.option(name, value);
                break;
            case "onMessageSend":
                this._createMessageSendAction();
                break;
            default:
                super._optionChanged(args)
        }
    }
    renderMessage() {
        let message = arguments.length > 0 && void 0 !== arguments[0] ? arguments[0] : {};
        const {
            items: items
        } = this.option();
        const newItems = [...items ?? [], message];
        this.option("items", newItems)
    }
}(0, _component_registrator.default)("dxChat", Chat);
var _default = exports.default = Chat;
