/**
 * DevExtreme (cjs/__internal/ui/chat/chat_message_group.js)
 * Version: 24.2.1-alpha-24256-1935
 * Build date: Thu Sep 12 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = void 0;
var _renderer = _interopRequireDefault(require("../../../core/renderer"));
var _date_serialization = _interopRequireDefault(require("../../../core/utils/date_serialization"));
var _type = require("../../../core/utils/type");
var _widget = _interopRequireDefault(require("../../core/widget/widget"));
var _chat_avatar = _interopRequireDefault(require("./chat_avatar"));
var _chat_message_bubble = _interopRequireDefault(require("./chat_message_bubble"));

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _extends() {
    _extends = Object.assign ? Object.assign.bind() : function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}
const CHAT_MESSAGE_GROUP_CLASS = "dx-chat-message-group";
const CHAT_MESSAGE_GROUP_ALIGNMENT_START_CLASS = "dx-chat-message-group-alignment-start";
const CHAT_MESSAGE_GROUP_ALIGNMENT_END_CLASS = "dx-chat-message-group-alignment-end";
const CHAT_MESSAGE_GROUP_INFORMATION_CLASS = "dx-chat-message-group-information";
const CHAT_MESSAGE_TIME_CLASS = "dx-chat-message-time";
const CHAT_MESSAGE_AUTHOR_NAME_CLASS = "dx-chat-message-author-name";
const CHAT_MESSAGE_BUBBLE_CONTAINER_CLASS = "dx-chat-message-bubble-container";
class MessageGroup extends _widget.default {
    _getDefaultOptions() {
        return _extends({}, super._getDefaultOptions(), {
            items: [],
            alignment: "start"
        })
    }
    _updateAlignmentClass() {
        const {
            alignment: alignment
        } = this.option();
        (0, _renderer.default)(this.element()).removeClass("dx-chat-message-group-alignment-start").removeClass("dx-chat-message-group-alignment-end");
        const alignmentClass = "start" === alignment ? "dx-chat-message-group-alignment-start" : "dx-chat-message-group-alignment-end";
        (0, _renderer.default)(this.element()).addClass(alignmentClass)
    }
    _initMarkup() {
        const {
            alignment: alignment,
            items: items
        } = this.option();
        (0, _renderer.default)(this.element()).addClass("dx-chat-message-group");
        this._updateAlignmentClass();
        super._initMarkup();
        if (0 === items.length) {
            return
        }
        if ("start" === alignment) {
            this._renderAvatar()
        }
        this._renderMessageGroupInformation(null === items || void 0 === items ? void 0 : items[0]);
        this._renderMessageBubbles(items)
    }
    _renderAvatar() {
        const $avatar = (0, _renderer.default)("<div>").appendTo(this.element());
        const {
            items: items
        } = this.option();
        const {
            author: author
        } = items[0];
        const authorName = null === author || void 0 === author ? void 0 : author.name;
        const authorAvatarUrl = null === author || void 0 === author ? void 0 : author.avatarUrl;
        this._avatar = this._createComponent($avatar, _chat_avatar.default, {
            name: authorName,
            url: authorAvatarUrl
        })
    }
    _renderMessageBubble(message) {
        const $bubble = (0, _renderer.default)("<div>");
        this._createComponent($bubble, _chat_message_bubble.default, {
            text: message.text
        });
        this._$messageBubbleContainer.append($bubble)
    }
    _renderMessageBubbles(items) {
        this._$messageBubbleContainer = (0, _renderer.default)("<div>").addClass("dx-chat-message-bubble-container");
        items.forEach((message => {
            this._renderMessageBubble(message)
        }));
        this._$messageBubbleContainer.appendTo(this.element())
    }
    _renderMessageGroupInformation(message) {
        const {
            alignment: alignment
        } = this.option();
        const {
            timestamp: timestamp,
            author: author
        } = message;
        const $information = (0, _renderer.default)("<div>").addClass("dx-chat-message-group-information");
        const authorName = (null === author || void 0 === author ? void 0 : author.name) ?? "";
        const authorNameText = "start" === alignment ? authorName : "";
        (0, _renderer.default)("<div>").addClass("dx-chat-message-author-name").text(authorNameText).appendTo($information);
        const $time = (0, _renderer.default)("<div>").addClass("dx-chat-message-time").appendTo($information);
        if ((0, _type.isDefined)(timestamp)) {
            $time.text(this._getTimeValue(timestamp))
        }
        $information.appendTo(this.element())
    }
    _getTimeValue(timestamp) {
        const date = _date_serialization.default.deserializeDate(timestamp);
        return date.toLocaleTimeString(void 0, {
            hour: "2-digit",
            minute: "2-digit",
            hour12: false
        })
    }
    _optionChanged(args) {
        const {
            name: name
        } = args;
        switch (name) {
            case "items":
            case "alignment":
                this._invalidate();
                break;
            default:
                super._optionChanged(args)
        }
    }
    renderMessage(message) {
        const {
            items: items
        } = this.option();
        const newItems = [...items, message];
        this._setOptionWithoutOptionChange("items", newItems);
        this._renderMessageBubble(message)
    }
}
var _default = exports.default = MessageGroup;
