/**
 * DevExtreme (esm/__internal/grids/grid_core/sticky_columns/dom.js)
 * Version: 24.2.1-alpha-24256-1935
 * Build date: Thu Sep 12 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    CLASSES,
    StickyPosition
} from "./const";
import {
    getColumnFixedPosition
} from "./utils";
const addStickyColumnBorderLeftClass = ($cell, addWidgetPrefix) => {
    $cell.addClass(addWidgetPrefix(CLASSES.stickyColumnBorderLeft))
};
const addStickyColumnBorderRightClass = ($cell, addWidgetPrefix) => {
    $cell.addClass(addWidgetPrefix(CLASSES.stickyColumnBorderRight))
};
const addStickyColumnClass = ($cell, stickyColumn, addWidgetPrefix) => {
    const fixedPosition = getColumnFixedPosition(stickyColumn);
    switch (fixedPosition) {
        case StickyPosition.Right:
            $cell.addClass(addWidgetPrefix(CLASSES.stickyColumnRight));
            break;
        case StickyPosition.Sticky:
            $cell.addClass(addWidgetPrefix(CLASSES.stickyColumn));
            break;
        default:
            $cell.addClass(addWidgetPrefix(CLASSES.stickyColumnLeft))
    }
};
const addFirstHeaderClass = ($cell, addWidgetPrefix) => {
    $cell.addClass(addWidgetPrefix(CLASSES.firstHeader))
};
const addColumnNoBorderClass = ($cell, addWidgetPrefix) => {
    $cell.addClass(addWidgetPrefix(CLASSES.columnNoBorder))
};
const toggleStickyColumnsClass = ($element, isStickyColumns, addWidgetPrefix) => {
    $element.toggleClass(addWidgetPrefix(CLASSES.stickyColumns), isStickyColumns)
};
export const GridCoreStickyColumnsDom = {
    addFirstHeaderClass: addFirstHeaderClass,
    addColumnNoBorderClass: addColumnNoBorderClass,
    addStickyColumnClass: addStickyColumnClass,
    addStickyColumnBorderLeftClass: addStickyColumnBorderLeftClass,
    addStickyColumnBorderRightClass: addStickyColumnBorderRightClass,
    toggleStickyColumnsClass: toggleStickyColumnsClass
};
