/**
 * DevExtreme (esm/__internal/pager/info.js)
 * Version: 24.2.1-alpha-24256-1935
 * Build date: Thu Sep 12 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    createVNode
} from "inferno";
import {
    BaseInfernoComponent
} from "@devextreme/runtime/inferno";
import {
    createRef as infernoCreateRef
} from "inferno";
import {
    format
} from "../../core/utils/string";
import messageLocalization from "../../localization/message";
import {
    PagerDefaultProps
} from "./common/pager_props";
export const PAGER_INFO_CLASS = "dx-info";
const InfoTextDefaultProps = {
    pageCount: PagerDefaultProps.pageCount,
    pageIndex: PagerDefaultProps.pageIndex,
    totalCount: PagerDefaultProps.totalCount
};
export class InfoText extends BaseInfernoComponent {
    constructor() {
        super(...arguments);
        this.state = {};
        this.refs = null;
        this.rootElementRef = infernoCreateRef()
    }
    getInfoText() {
        return this.props.infoText ?? messageLocalization.getFormatter("dxPager-infoText")()
    }
    getText() {
        const {
            pageCount: pageCount,
            pageIndex: pageIndex,
            totalCount: totalCount
        } = this.props;
        return format(this.getInfoText(), (pageIndex + 1).toString(), null === pageCount || void 0 === pageCount ? void 0 : pageCount.toString(), null === totalCount || void 0 === totalCount ? void 0 : totalCount.toString())
    }
    render() {
        return createVNode(1, "div", "dx-info", this.getText(), 0, null, null, this.props.rootElementRef)
    }
}
InfoText.defaultProps = InfoTextDefaultProps;
