/**
 * DevExtreme (esm/__internal/ui/chat/chat.js)
 * Version: 24.2.1-alpha-24256-1935
 * Build date: Thu Sep 12 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
import registerComponent from "../../../core/component_registrator";
import Guid from "../../../core/guid";
import $ from "../../../core/renderer";
import Widget from "../../core/widget/widget";
import ChatHeader from "./chat_header";
import MessageBox from "./chat_message_box";
import MessageList from "./chat_message_list";
const CHAT_CLASS = "dx-chat";
const TEXTEDITOR_INPUT_CLASS = "dx-texteditor-input";
class Chat extends Widget {
    _getDefaultOptions() {
        return _extends({}, super._getDefaultOptions(), {
            activeStateEnabled: true,
            focusStateEnabled: true,
            hoverStateEnabled: true,
            title: "",
            items: [],
            dataSource: null,
            user: {
                id: (new Guid).toString()
            },
            onMessageSend: void 0
        })
    }
    _init() {
        super._init();
        this._createMessageSendAction()
    }
    _initMarkup() {
        $(this.element()).addClass("dx-chat");
        super._initMarkup();
        this._renderMessageBox();
        this._renderMessageList();
        const {
            title: title
        } = this.option();
        if (title) {
            this._renderHeader(title)
        }
    }
    _renderHeader(title) {
        const $header = $("<div>");
        this.element().prepend($header.get(0));
        this._chatHeader = this._createComponent($header, ChatHeader, {
            title: title
        })
    }
    _renderMessageList() {
        const {
            items: items = [],
            user: user
        } = this.option();
        const currentUserId = null === user || void 0 === user ? void 0 : user.id;
        const $messageList = $("<div>");
        this.$element().prepend($messageList);
        this._messageList = this._createComponent($messageList, MessageList, {
            items: items,
            currentUserId: currentUserId
        })
    }
    _renderMessageBox() {
        const {
            activeStateEnabled: activeStateEnabled,
            focusStateEnabled: focusStateEnabled,
            hoverStateEnabled: hoverStateEnabled
        } = this.option();
        const $messageBox = $("<div>").appendTo(this.element());
        const configuration = {
            activeStateEnabled: activeStateEnabled,
            focusStateEnabled: focusStateEnabled,
            hoverStateEnabled: hoverStateEnabled,
            onMessageSend: e => {
                this._messageSendHandler(e)
            }
        };
        this._messageBox = this._createComponent($messageBox, MessageBox, configuration)
    }
    _createMessageSendAction() {
        this._messageSendAction = this._createActionByOption("onMessageSend", {
            excludeValidators: ["disabled", "readOnly"]
        })
    }
    _messageSendHandler(e) {
        var _this$_messageSendAct;
        const {
            text: text,
            event: event
        } = e;
        const {
            user: user
        } = this.option();
        const message = {
            timestamp: new Date,
            author: user,
            text: text
        };
        this.renderMessage(message);
        null === (_this$_messageSendAct = this._messageSendAction) || void 0 === _this$_messageSendAct || _this$_messageSendAct.call(this, {
            message: message,
            event: event
        })
    }
    _focusTarget() {
        const $input = $(this.element()).find(".dx-texteditor-input");
        return $input
    }
    _optionChanged(args) {
        const {
            name: name,
            value: value
        } = args;
        switch (name) {
            case "activeStateEnabled":
            case "focusStateEnabled":
            case "hoverStateEnabled":
                this._messageBox.option(name, value);
                break;
            case "title":
                if (value) {
                    if (this._chatHeader) {
                        this._chatHeader.option("title", value)
                    } else {
                        this._renderHeader(value)
                    }
                } else if (this._chatHeader) {
                    this._chatHeader.dispose();
                    this._chatHeader.$element().remove()
                }
                break;
            case "user": {
                const author = value;
                this._messageList.option("currentUserId", null === author || void 0 === author ? void 0 : author.id);
                break
            }
            case "items":
            case "dataSource":
                this._messageList.option(name, value);
                break;
            case "onMessageSend":
                this._createMessageSendAction();
                break;
            default:
                super._optionChanged(args)
        }
    }
    renderMessage() {
        let message = arguments.length > 0 && void 0 !== arguments[0] ? arguments[0] : {};
        const {
            items: items
        } = this.option();
        const newItems = [...items ?? [], message];
        this.option("items", newItems)
    }
}
registerComponent("dxChat", Chat);
export default Chat;
