/**
 * DevExtreme (esm/__internal/ui/chat/chat_avatar.js)
 * Version: 24.2.1-alpha-24256-1935
 * Build date: Thu Sep 12 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
import $ from "../../../core/renderer";
import {
    isDefined
} from "../../../core/utils/type";
import Widget from "../../core/widget/widget";
const CHAT_MESSAGE_AVATAR_CLASS = "dx-chat-message-avatar";
const CHAT_MESSAGE_AVATAR_INITIALS_CLASS = "dx-chat-message-avatar-initials";
const CHAT_MESSAGE_AVATAR_IMAGE_CLASS = "dx-chat-message-avatar-image";
class Avatar extends Widget {
    _getDefaultOptions() {
        return _extends({}, super._getDefaultOptions(), {
            name: "",
            url: ""
        })
    }
    _initMarkup() {
        $(this.element()).addClass("dx-chat-message-avatar");
        super._initMarkup();
        this._renderAvatarContent()
    }
    _renderAvatarContent() {
        var _this$_$content;
        null === (_this$_$content = this._$content) || void 0 === _this$_$content || _this$_$content.remove();
        if (this._isValuableUrl()) {
            this._renderImage();
            return
        }
        this._renderInitials()
    }
    _renderImage() {
        this._renderImageElement();
        this._updateUrl();
        this._updateAlt()
    }
    _renderInitials() {
        const {
            name: name
        } = this.option();
        if (name) {
            this._renderInitialsElement();
            this._updateInitials()
        }
    }
    _renderImageElement() {
        this._$content = $("<img>").addClass("dx-chat-message-avatar-image").appendTo(this.element())
    }
    _renderInitialsElement() {
        this._$content = $("<div>").addClass("dx-chat-message-avatar-initials").appendTo(this.element())
    }
    _updateInitials() {
        var _this$_$content2;
        const {
            name: name
        } = this.option();
        null === (_this$_$content2 = this._$content) || void 0 === _this$_$content2 || _this$_$content2.text(this._getInitials(name))
    }
    _updateUrl() {
        var _this$_$content3;
        const {
            url: url
        } = this.option();
        null === (_this$_$content3 = this._$content) || void 0 === _this$_$content3 || _this$_$content3.attr("src", url ?? "")
    }
    _updateAlt() {
        var _this$_$content4;
        const {
            name: name
        } = this.option();
        null === (_this$_$content4 = this._$content) || void 0 === _this$_$content4 || _this$_$content4.attr("alt", name || "Avatar")
    }
    _isValuableUrl() {
        var _url$trim;
        const {
            url: url
        } = this.option();
        const result = !!(null !== url && void 0 !== url && null !== (_url$trim = url.trim) && void 0 !== _url$trim && _url$trim.call(url));
        return result
    }
    _getInitials(name) {
        if (isDefined(name)) {
            return String(name).charAt(0).toUpperCase()
        }
        return ""
    }
    _optionChanged(args) {
        const {
            name: name
        } = args;
        switch (name) {
            case "name":
            case "url":
                this._renderAvatarContent();
                break;
            default:
                super._optionChanged(args)
        }
    }
}
export default Avatar;
