/**
 * DevExtreme (esm/__internal/ui/chat/chat_message_box.js)
 * Version: 24.2.1-alpha-24256-1935
 * Build date: Thu Sep 12 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
import $ from "../../../core/renderer";
import Button from "../../../ui/button";
import DOMComponent from "../../core/widget/dom_component";
import TextArea from "../m_text_area";
const CHAT_MESSAGE_BOX_CLASS = "dx-chat-message-box";
const CHAT_MESSAGE_BOX_TEXTAREA_CLASS = "dx-chat-message-box-text-area";
const CHAT_MESSAGE_BOX_BUTTON_CLASS = "dx-chat-message-box-button";
class MessageBox extends DOMComponent {
    _getDefaultOptions() {
        return _extends({}, super._getDefaultOptions(), {
            onMessageSend: void 0,
            activeStateEnabled: true,
            focusStateEnabled: true,
            hoverStateEnabled: true
        })
    }
    _init() {
        super._init();
        this._createMessageSendAction()
    }
    _initMarkup() {
        $(this.element()).addClass("dx-chat-message-box");
        super._initMarkup();
        this._renderTextArea();
        this._renderButton()
    }
    _isValuableTextEntered() {
        const {
            text: text
        } = this._textArea.option();
        return !!(null !== text && void 0 !== text && text.trim())
    }
    _renderTextArea() {
        const {
            activeStateEnabled: activeStateEnabled,
            focusStateEnabled: focusStateEnabled,
            hoverStateEnabled: hoverStateEnabled
        } = this.option();
        const $textArea = $("<div>").addClass("dx-chat-message-box-text-area").appendTo(this.element());
        this._textArea = this._createComponent($textArea, TextArea, {
            activeStateEnabled: activeStateEnabled,
            focusStateEnabled: focusStateEnabled,
            hoverStateEnabled: hoverStateEnabled,
            stylingMode: "outlined",
            placeholder: "Type a message",
            autoResizeEnabled: true,
            valueChangeEvent: "input",
            maxHeight: "20em",
            onInput: () => {
                const shouldButtonBeDisabled = !this._isValuableTextEntered();
                this._toggleButtonDisableState(shouldButtonBeDisabled)
            },
            onEnterKey: e => {
                var _e$event;
                if (!(null !== (_e$event = e.event) && void 0 !== _e$event && _e$event.shiftKey)) {
                    this._sendHandler(e)
                }
            }
        });
        this._textArea.registerKeyHandler("enter", (event => {
            if (!event.shiftKey && this._isValuableTextEntered()) {
                event.preventDefault()
            }
        }))
    }
    _renderButton() {
        const {
            activeStateEnabled: activeStateEnabled,
            focusStateEnabled: focusStateEnabled,
            hoverStateEnabled: hoverStateEnabled
        } = this.option();
        const $button = $("<div>").addClass("dx-chat-message-box-button").appendTo(this.element());
        this._button = this._createComponent($button, Button, {
            activeStateEnabled: activeStateEnabled,
            focusStateEnabled: focusStateEnabled,
            hoverStateEnabled: hoverStateEnabled,
            icon: "sendfilled",
            type: "default",
            stylingMode: "text",
            disabled: true,
            onClick: e => {
                this._sendHandler(e)
            }
        })
    }
    _createMessageSendAction() {
        this._messageSendAction = this._createActionByOption("onMessageSend", {
            excludeValidators: ["disabled", "readOnly"]
        })
    }
    _sendHandler(e) {
        var _this$_messageSendAct;
        if (!this._isValuableTextEntered()) {
            return
        }
        const {
            text: text
        } = this._textArea.option();
        null === (_this$_messageSendAct = this._messageSendAction) || void 0 === _this$_messageSendAct || _this$_messageSendAct.call(this, {
            text: text,
            event: e.event
        });
        this._textArea.reset();
        this._toggleButtonDisableState(true)
    }
    _toggleButtonDisableState(state) {
        this._button.option("disabled", state)
    }
    _optionChanged(args) {
        const {
            name: name,
            value: value
        } = args;
        switch (name) {
            case "activeStateEnabled":
            case "focusStateEnabled":
            case "hoverStateEnabled":
                this._button.option(name, value);
                this._textArea.option(name, value);
                break;
            case "onMessageSend":
                this._createMessageSendAction();
                break;
            default:
                super._optionChanged(args)
        }
    }
}
export default MessageBox;
