/**
 * DevExtreme (esm/__internal/ui/chat/chat_message_group.js)
 * Version: 24.2.1-alpha-24256-1935
 * Build date: Thu Sep 12 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
import $ from "../../../core/renderer";
import dateSerialization from "../../../core/utils/date_serialization";
import {
    isDefined
} from "../../../core/utils/type";
import Widget from "../../core/widget/widget";
import Avatar from "./chat_avatar";
import MessageBubble from "./chat_message_bubble";
const CHAT_MESSAGE_GROUP_CLASS = "dx-chat-message-group";
const CHAT_MESSAGE_GROUP_ALIGNMENT_START_CLASS = "dx-chat-message-group-alignment-start";
const CHAT_MESSAGE_GROUP_ALIGNMENT_END_CLASS = "dx-chat-message-group-alignment-end";
const CHAT_MESSAGE_GROUP_INFORMATION_CLASS = "dx-chat-message-group-information";
const CHAT_MESSAGE_TIME_CLASS = "dx-chat-message-time";
const CHAT_MESSAGE_AUTHOR_NAME_CLASS = "dx-chat-message-author-name";
const CHAT_MESSAGE_BUBBLE_CONTAINER_CLASS = "dx-chat-message-bubble-container";
class MessageGroup extends Widget {
    _getDefaultOptions() {
        return _extends({}, super._getDefaultOptions(), {
            items: [],
            alignment: "start"
        })
    }
    _updateAlignmentClass() {
        const {
            alignment: alignment
        } = this.option();
        $(this.element()).removeClass("dx-chat-message-group-alignment-start").removeClass("dx-chat-message-group-alignment-end");
        const alignmentClass = "start" === alignment ? "dx-chat-message-group-alignment-start" : "dx-chat-message-group-alignment-end";
        $(this.element()).addClass(alignmentClass)
    }
    _initMarkup() {
        const {
            alignment: alignment,
            items: items
        } = this.option();
        $(this.element()).addClass("dx-chat-message-group");
        this._updateAlignmentClass();
        super._initMarkup();
        if (0 === items.length) {
            return
        }
        if ("start" === alignment) {
            this._renderAvatar()
        }
        this._renderMessageGroupInformation(null === items || void 0 === items ? void 0 : items[0]);
        this._renderMessageBubbles(items)
    }
    _renderAvatar() {
        const $avatar = $("<div>").appendTo(this.element());
        const {
            items: items
        } = this.option();
        const {
            author: author
        } = items[0];
        const authorName = null === author || void 0 === author ? void 0 : author.name;
        const authorAvatarUrl = null === author || void 0 === author ? void 0 : author.avatarUrl;
        this._avatar = this._createComponent($avatar, Avatar, {
            name: authorName,
            url: authorAvatarUrl
        })
    }
    _renderMessageBubble(message) {
        const $bubble = $("<div>");
        this._createComponent($bubble, MessageBubble, {
            text: message.text
        });
        this._$messageBubbleContainer.append($bubble)
    }
    _renderMessageBubbles(items) {
        this._$messageBubbleContainer = $("<div>").addClass("dx-chat-message-bubble-container");
        items.forEach((message => {
            this._renderMessageBubble(message)
        }));
        this._$messageBubbleContainer.appendTo(this.element())
    }
    _renderMessageGroupInformation(message) {
        const {
            alignment: alignment
        } = this.option();
        const {
            timestamp: timestamp,
            author: author
        } = message;
        const $information = $("<div>").addClass("dx-chat-message-group-information");
        const authorName = (null === author || void 0 === author ? void 0 : author.name) ?? "";
        const authorNameText = "start" === alignment ? authorName : "";
        $("<div>").addClass("dx-chat-message-author-name").text(authorNameText).appendTo($information);
        const $time = $("<div>").addClass("dx-chat-message-time").appendTo($information);
        if (isDefined(timestamp)) {
            $time.text(this._getTimeValue(timestamp))
        }
        $information.appendTo(this.element())
    }
    _getTimeValue(timestamp) {
        const date = dateSerialization.deserializeDate(timestamp);
        return date.toLocaleTimeString(void 0, {
            hour: "2-digit",
            minute: "2-digit",
            hour12: false
        })
    }
    _optionChanged(args) {
        const {
            name: name
        } = args;
        switch (name) {
            case "items":
            case "alignment":
                this._invalidate();
                break;
            default:
                super._optionChanged(args)
        }
    }
    renderMessage(message) {
        const {
            items: items
        } = this.option();
        const newItems = [...items, message];
        this._setOptionWithoutOptionChange("items", newItems);
        this._renderMessageBubble(message)
    }
}
export default MessageGroup;
