/**
 * DevExtreme (esm/__internal/ui/chat/chat_message_list.js)
 * Version: 24.2.1-alpha-24256-1935
 * Build date: Thu Sep 12 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
import $ from "../../../core/renderer";
import {
    hasWindow
} from "../../../core/utils/window";
import messageLocalization from "../../../localization/message";
import Scrollable from "../../../ui/scroll_view/ui.scrollable";
import Widget from "../../core/widget/widget";
import MessageGroup from "./chat_message_group";
const CHAT_MESSAGE_LIST_CLASS = "dx-chat-message-list";
const CHAT_MESSAGE_LIST_EMPTY_CLASS = "dx-chat-message-list-empty";
const CHAT_MESSAGE_LIST_EMPTY_VIEW_CLASS = "dx-chat-empty-view";
const CHAT_MESSAGE_LIST_EMPTY_IMAGE_CLASS = "dx-chat-empty-image";
const CHAT_MESSAGE_LIST_EMPTY_MESSAGE_CLASS = "dx-chat-empty-message";
const CHAT_MESSAGE_LIST_EMPTY_PROMPT_CLASS = "dx-chat-empty-prompt";
class MessageList extends Widget {
    _getDefaultOptions() {
        return _extends({}, super._getDefaultOptions(), {
            items: [],
            currentUserId: ""
        })
    }
    _init() {
        super._init();
        this._messageGroups = []
    }
    _initMarkup() {
        $(this.element()).addClass("dx-chat-message-list");
        super._initMarkup();
        this._renderScrollable();
        this._renderMessageListContent();
        this._scrollable.update();
        this._scrollContentToLastMessageGroup()
    }
    _renderEmptyViewContent() {
        const $emptyView = $("<div>").addClass("dx-chat-empty-view");
        $("<div>").appendTo($emptyView).addClass("dx-chat-empty-image");
        const messageText = messageLocalization.format("dxChat-emptyListMessage");
        $("<div>").appendTo($emptyView).addClass("dx-chat-empty-message").text(messageText);
        const promptText = messageLocalization.format("dxChat-emptyListPrompt");
        $("<div>").appendTo($emptyView).addClass("dx-chat-empty-prompt").text(promptText);
        $emptyView.appendTo(this._scrollable.content())
    }
    _removeEmptyView() {
        $(this._scrollable.content()).empty()
    }
    _toggleEmptyStateClass(state) {
        this.$element().toggleClass("dx-chat-message-list-empty", state)
    }
    _isEmpty() {
        const {
            items: items
        } = this.option();
        return 0 === items.length
    }
    _isCurrentUser(id) {
        const {
            currentUserId: currentUserId
        } = this.option();
        return currentUserId === id
    }
    _messageGroupAlignment(id) {
        return this._isCurrentUser(id) ? "end" : "start"
    }
    _createMessageGroupComponent(items, userId) {
        var _this$_messageGroups;
        const $messageGroup = $("<div>").appendTo(this._scrollable.content());
        const messageGroup = this._createComponent($messageGroup, MessageGroup, {
            items: items,
            alignment: this._messageGroupAlignment(userId)
        });
        null === (_this$_messageGroups = this._messageGroups) || void 0 === _this$_messageGroups || _this$_messageGroups.push(messageGroup)
    }
    _renderScrollable() {
        const $scrollable = $("<div>").appendTo(this.$element());
        this._scrollable = this._createComponent($scrollable, Scrollable, {
            useNative: true
        })
    }
    _renderMessageListContent() {
        var _items$;
        if (this._isEmpty()) {
            this._toggleEmptyStateClass(true);
            this._renderEmptyViewContent();
            return
        }
        const {
            items: items
        } = this.option();
        let currentMessageGroupUserId = null === (_items$ = items[0]) || void 0 === _items$ || null === (_items$ = _items$.author) || void 0 === _items$ ? void 0 : _items$.id;
        let currentMessageGroupItems = [];
        items.forEach(((item, index) => {
            var _newMessageGroupItem$;
            const newMessageGroupItem = item ?? {};
            const id = null === (_newMessageGroupItem$ = newMessageGroupItem.author) || void 0 === _newMessageGroupItem$ ? void 0 : _newMessageGroupItem$.id;
            if (id === currentMessageGroupUserId) {
                currentMessageGroupItems.push(newMessageGroupItem)
            } else {
                this._createMessageGroupComponent(currentMessageGroupItems, currentMessageGroupUserId);
                currentMessageGroupUserId = id;
                currentMessageGroupItems = [];
                currentMessageGroupItems.push(newMessageGroupItem)
            }
            if (items.length - 1 === index) {
                this._createMessageGroupComponent(currentMessageGroupItems, currentMessageGroupUserId)
            }
        }))
    }
    _renderMessage(message) {
        var _this$_messageGroups2;
        const sender = message.author;
        const lastMessageGroup = null === (_this$_messageGroups2 = this._messageGroups) || void 0 === _this$_messageGroups2 ? void 0 : _this$_messageGroups2[this._messageGroups.length - 1];
        if (lastMessageGroup) {
            var _lastMessageGroup$opt;
            const lastMessageGroupUserId = null === (_lastMessageGroup$opt = lastMessageGroup.option("items")[0].author) || void 0 === _lastMessageGroup$opt ? void 0 : _lastMessageGroup$opt.id;
            if ((null === sender || void 0 === sender ? void 0 : sender.id) === lastMessageGroupUserId) {
                lastMessageGroup.renderMessage(message);
                this._scrollContentToLastMessageGroup();
                return
            }
        }
        this._createMessageGroupComponent([message], null === sender || void 0 === sender ? void 0 : sender.id);
        this._scrollContentToLastMessageGroup()
    }
    _scrollContentToLastMessageGroup() {
        var _this$_messageGroups3;
        if (!(null !== (_this$_messageGroups3 = this._messageGroups) && void 0 !== _this$_messageGroups3 && _this$_messageGroups3.length && hasWindow())) {
            return
        }
        const lastMessageGroup = this._messageGroups[this._messageGroups.length - 1];
        const element = lastMessageGroup.$element()[0];
        this._scrollable.scrollToElement(element)
    }
    _clean() {
        this._messageGroups = [];
        super._clean()
    }
    _isMessageAddedToEnd(value, previousValue) {
        const valueLength = value.length;
        const previousValueLength = previousValue.length;
        if (0 === valueLength) {
            return false
        }
        if (0 === previousValueLength) {
            return 1 === valueLength
        }
        const lastValueItem = value[valueLength - 1];
        const lastPreviousValueItem = previousValue[previousValueLength - 1];
        const isLastItemNotTheSame = lastValueItem !== lastPreviousValueItem;
        const isLengthIncreasedByOne = valueLength - previousValueLength === 1;
        return isLastItemNotTheSame && isLengthIncreasedByOne
    }
    _processItemsUpdating(value, previousValue) {
        const shouldItemsBeUpdatedCompletely = !this._isMessageAddedToEnd(value, previousValue);
        if (shouldItemsBeUpdatedCompletely) {
            this._invalidate()
        } else {
            this._toggleEmptyStateClass(false);
            if (!previousValue.length) {
                this._removeEmptyView()
            }
            const newMessage = value[value.length - 1];
            this._renderMessage(newMessage ?? {})
        }
    }
    _optionChanged(args) {
        const {
            name: name,
            value: value,
            previousValue: previousValue
        } = args;
        switch (name) {
            case "currentUserId":
                this._invalidate();
                break;
            case "items":
                this._processItemsUpdating(value ?? [], previousValue ?? []);
                break;
            default:
                super._optionChanged(args)
        }
    }
}
export default MessageList;
