/**
* DevExtreme (ui/chat.d.ts)
* Version: 24.2.1-alpha-24256-1935
* Build date: Thu Sep 12 2024
*
* Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import Widget, { WidgetOptions } from './widget/ui.widget';
import {
    EventInfo,
    NativeEventInfo,
    InitializedEventInfo,
    ChangedOptionInfo,
} from '../events/index';
import DataSource, { DataSourceLike } from '../data/data_source';

/**
 * 
 */
export type DisposingEvent = EventInfo<dxChat>;

/**
 * 
 */
export type InitializedEvent = InitializedEventInfo<dxChat>;

/**
 * 
 */
export type OptionChangedEvent = EventInfo<dxChat> & ChangedOptionInfo;

/**
 * 
 */
export type MessageSendEvent = NativeEventInfo<dxChat, KeyboardEvent | PointerEvent | MouseEvent | TouchEvent> & {
    /**
     * 
     */
    readonly message?: Message;
};

/**
 * 
 */
export type User = {
    /**
     * 
     */
    id?: number | string;
    /**
     * 
     */
    name?: string;
    /**
     * 
     */
    avatarUrl?: string;
};

/**
 * 
 */
export type Message = {
    /**
     * 
     */
    timestamp?: Date | number | string;
    /**
     * 
     */
    author?: User;
    /**
     * 
     */
    text?: string;
    /**
     * 
     */
    typing?: boolean;
};

/**
 * 
 * @deprecated 
 */
export interface dxChatOptions extends WidgetOptions<dxChat> {
    /**
     * 
     */
    activeStateEnabled?: boolean;
    /**
     * 
     */
    focusStateEnabled?: boolean;
    /**
     * 
     */
    hoverStateEnabled?: boolean;
    /**
     * 
     */
    user?: User;
    /**
     * 
     */
    items?: Array<Message>;
    /**
     * 
     */
    dataSource?: DataSourceLike<Message> | null;
    /**
     * 
     */
    onMessageSend?: ((e: MessageSendEvent) => void);
}

/**
 * 
 */
export default class dxChat extends Widget<Properties> {
    /**
     * 
     */
    renderMessage(message: Message): void;

    getDataSource(): DataSource<Message>;
}

export type ExplicitTypes = {
    Properties: Properties;
    DisposingEvent: DisposingEvent;
    InitializedEvent: InitializedEvent;
    OptionChangedEvent: OptionChangedEvent;
};

export type Properties = dxChatOptions;


